@@ -17,7 +17,7 @@ import signal
 import greenlet
 import logging
 
-from bpython._py3compat import py3, is_main_thread
+from bpython._py3compat import py3
 from bpython.config import getpreferredencoding
 
 logger = logging.getLogger(__name__)
@@ -85,7 +85,6 @@ class CodeRunner(object):
     just passes whatever is passed in to run_code(for_code) to the
     code greenlet
     """
-
     def __init__(self, interp=None, request_refresh=lambda: None):
         """
         interp is an interpreter object to use. By default a new one is
@@ -113,9 +112,8 @@ class CodeRunner(object):
 
     def load_code(self, source):
         """Prep code to be run"""
-        assert self.source is None, (
-            "you shouldn't load code when some is " "already running"
-        )
+        assert self.source is None, "you shouldn't load code when some is " \
+            "already running"
         self.source = source
         self.code_context = None
 
@@ -135,26 +133,23 @@ class CodeRunner(object):
         if self.code_context is None:
             assert self.source is not None
             self.code_context = greenlet.greenlet(self._blocking_run_code)
-            if is_main_thread():
-                self.orig_sigint_handler = signal.getsignal(signal.SIGINT)
-                signal.signal(signal.SIGINT, self.sigint_handler)
+            self.orig_sigint_handler = signal.getsignal(signal.SIGINT)
+            signal.signal(signal.SIGINT, self.sigint_handler)
             request = self.code_context.switch()
         else:
             assert self.code_is_waiting
             self.code_is_waiting = False
-            if is_main_thread():
-                signal.signal(signal.SIGINT, self.sigint_handler)
+            signal.signal(signal.SIGINT, self.sigint_handler)
             if self.sigint_happened_in_main_context:
                 self.sigint_happened_in_main_context = False
                 request = self.code_context.switch(SigintHappened)
             else:
                 request = self.code_context.switch(for_code)
 
-        logger.debug("request received from code was %r", request)
+        logger.debug('request received from code was %r', request)
         if not isinstance(request, RequestFromCodeRunner):
-            raise ValueError(
-                "Not a valid value from code greenlet: %r" % request
-            )
+            raise ValueError("Not a valid value from code greenlet: %r" %
+                             request)
         if isinstance(request, (Wait, Refresh)):
             self.code_is_waiting = True
             if isinstance(request, Refresh):
@@ -162,8 +157,7 @@ class CodeRunner(object):
             return False
         elif isinstance(request, (Done, Unfinished)):
             self._unload_code()
-            if is_main_thread():
-                signal.signal(signal.SIGINT, self.orig_sigint_handler)
+            signal.signal(signal.SIGINT, self.orig_sigint_handler)
             self.orig_sigint_handler = None
             return request
         elif isinstance(request, SystemExitRequest):
@@ -174,20 +168,18 @@ class CodeRunner(object):
         """SIGINT handler to use while code is running or request being
         fulfilled"""
         if greenlet.getcurrent() is self.code_context:
-            logger.debug("sigint while running user code!")
+            logger.debug('sigint while running user code!')
             raise KeyboardInterrupt()
         else:
-            logger.debug(
-                "sigint while fulfilling code request sigint handler "
-                "running!"
-            )
+            logger.debug('sigint while fulfilling code request sigint handler '
+                         'running!')
             self.sigint_happened_in_main_context = True
 
     def _blocking_run_code(self):
         try:
             unfinished = self.interp.runsource(self.source)
         except SystemExit as e:
-            return SystemExitRequest(*e.args)
+            return SystemExitRequest(e.args)
         return Unfinished() if unfinished else Done()
 
     def request_from_main_context(self, force_refresh=False):
@@ -205,7 +197,7 @@ class CodeRunner(object):
 
 
 class FakeOutput(object):
-    def __init__(self, coderunner, on_write, real_fileobj):
+    def __init__(self, coderunner, on_write, fileno=1):
         """Fakes sys.stdout or sys.stderr
 
         on_write should always take unicode
@@ -215,11 +207,11 @@ class FakeOutput(object):
         """
         self.coderunner = coderunner
         self.on_write = on_write
-        self._real_fileobj = real_fileobj
+        self.real_fileno = fileno
 
     def write(self, s, *args, **kwargs):
         if not py3 and isinstance(s, str):
-            s = s.decode(getpreferredencoding(), "ignore")
+            s = s.decode(getpreferredencoding(), 'ignore')
         self.on_write(s, *args, **kwargs)
         return self.coderunner.request_from_main_context(force_refresh=True)
 
@@ -227,7 +219,7 @@ class FakeOutput(object):
     # have a method called fileno. One example is pwntools. This
     # is not a widespread issue, but is annoying.
     def fileno(self):
-        return self._real_fileobj.fileno()
+        return self.real_fileno
 
     def writelines(self, l):
         for s in l:
@@ -238,7 +230,3 @@ class FakeOutput(object):
 
     def isatty(self):
         return True
-
-    @property
-    def encoding(self):
-        return self._real_fileobj.encoding
